use std::{num::ParseIntError, str::FromStr};

#[derive(Clone, Debug, Default)]
pub(crate) struct Revision {
    pub(crate) version: String,
    pub(crate) revision: usize,
}

#[derive(Debug)]
pub(crate) enum RevisionError {
    Revision(ParseIntError),
}

impl Revision {
    pub(crate) fn next(self, version_string: Option<String>) -> Self {
        if let Some(version_string) = version_string {
            if self.version == version_string {
                Revision {
                    revision: self.revision + 1,
                    ..self
                }
            } else {
                Revision {
                    version: version_string,
                    revision: 0,
                }
            }
        } else {
            Revision {
                revision: self.revision + 1,
                ..self
            }
        }
    }
}

impl FromStr for Revision {
    type Err = RevisionError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        if let Some((version, revision)) = s.rsplit_once('-') {
            let revision = revision
                .trim_start_matches('r')
                .parse()
                .map_err(RevisionError::Revision)?;

            Ok(Revision {
                version: version.to_owned(),
                revision,
            })
        } else {
            Ok(Revision {
                version: s.to_string(),
                revision: 0,
            })
        }
    }
}

impl std::fmt::Display for Revision {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}-r{}", self.version, self.revision)
    }
}

impl std::fmt::Display for RevisionError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::Revision(e) => write!(f, "Failed to parse revision number, {}", e),
        }
    }
}

impl std::error::Error for RevisionError {}
